@@ -390,33 +390,63 @@ impl Rest {
             .await
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub async fn place_order(
         &self,
         market: &str,
         side: OrderSide,
-        price: Option<f64>,
+        price: Option<Decimal>,
         r#type: OrderType,
-        size: f64,
+        size: Decimal,
+        reduce_only: Option<bool>,
+        ioc: Option<bool>,
+        post_only: Option<bool>,
         client_id: Option<&str>,
     ) -> Result<OrderInfo> {
+        // Limit orders should have price specified
+        if let OrderType::Limit = r#type {
+            if price.is_none() {
+                return Err(Error::PlacingLimitOrderRequiresPrice);
+            }
+        }
+
         self.post(
             "/orders",
             Some(json!({
                 "market": market,
                 "side": side,
-                "price": price,
+                // As per docs, send null for market orders
+                "price": if let OrderType::Limit = r#type { price } else { None },
                 "type": r#type,
                 "size": size,
-                "reduceOnly": false,
-                "ioc": false,
-                "postOnly": false,
+                "reduceOnly": reduce_only.unwrap_or(false),
+                "ioc": ioc.unwrap_or(false),
+                "postOnly": post_only.unwrap_or(false),
                 "clientId": client_id,
             })),
         )
         .await
     }
 
-    pub async fn get_order(&self, order_id: usize) -> Result<OrderInfo> {
+    pub async fn modify_order(
+        &self,
+        order_id: Id,
+        price: Option<Decimal>,
+        size: Option<Decimal>,
+        client_id: Option<&str>,
+    ) -> Result<OrderInfo> {
+        self.post(
+            format!("/orders/{}/modify", order_id).as_str(),
+            Some(json!({
+                "price": price,
+                "size": size,
+                "clientId": client_id,
+            })),
+        )
+        .await
+    }
+
+    pub async fn get_order(&self, order_id: Id) -> Result<OrderInfo> {
         self.get(&format!("/orders/{}", order_id), None).await
     }
 
@@ -425,7 +455,7 @@ impl Rest {
             .await
     }
 
-    pub async fn cancel_order(&self, order_id: usize) -> Result<String> {
+    pub async fn cancel_order(&self, order_id: Id) -> Result<String> {
         self.delete(&format!("/orders/{}", order_id), None).await
     }
 
