@@ -6,7 +6,7 @@ pub type Id = u64;
 pub type Coin = String;
 pub type Symbol = String;
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 #[serde(untagged)]
 pub enum Response<T> {
@@ -16,7 +16,7 @@ pub enum Response<T> {
 
 // REST API -> Subaccounts
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Subaccount {
     pub nickname: String,
@@ -27,7 +27,7 @@ pub struct Subaccount {
 
 pub type Subaccounts = Vec<Subaccount>;
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Create {
     pub nickname: String,
@@ -35,15 +35,15 @@ pub struct Create {
     pub editable: bool,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Copy, Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct ChangeName;
 
-#[derive(Debug, Deserialize)]
+#[derive(Copy, Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Delete;
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Balance {
     pub coin: Coin,
@@ -55,7 +55,7 @@ pub struct Balance {
 
 pub type Balances = Vec<Balance>;
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Transfer {
     pub id: Id,
@@ -67,14 +67,14 @@ pub struct Transfer {
 
 // REST API -> Markets
 
-#[derive(Debug, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum MarketType {
     Future,
     Spot,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Market {
     #[serde(rename = "type")]
@@ -103,21 +103,21 @@ pub struct Market {
 
 pub type Markets = Vec<Market>;
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Orderbook {
     pub asks: Vec<(Decimal, Decimal)>,
     pub bids: Vec<(Decimal, Decimal)>,
 }
 
-#[derive(Debug, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum Side {
     Buy,
     Sell,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Copy, Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Trade {
     pub id: Id,
@@ -130,7 +130,7 @@ pub struct Trade {
 
 pub type Trades = Vec<Trade>;
 
-#[derive(Debug, Deserialize)]
+#[derive(Copy, Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Price {
     pub close: Decimal,
@@ -145,7 +145,7 @@ pub type Prices = Vec<Price>;
 
 // REST API -> Futures
 
-#[derive(Debug, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum FutureType {
     Future,
@@ -154,7 +154,7 @@ pub enum FutureType {
     Move,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Future {
     pub ask: Option<Decimal>,
@@ -187,7 +187,7 @@ pub struct Future {
 
 pub type Futures = Vec<Future>;
 
-#[derive(Debug, Deserialize)]
+#[derive(Copy, Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct FutureStats {
     pub volume: Decimal,
@@ -199,7 +199,7 @@ pub struct FutureStats {
     pub open_interest: Decimal,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct FundingRate {
     pub future: Symbol,
@@ -213,7 +213,7 @@ pub type FundingRates = Vec<FundingRate>;
 
 /// Returned by GET /account.
 /// See https://docs.ftx.com/#get-account-information.
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Account {
     pub backstop_provider: bool,
@@ -243,7 +243,7 @@ pub struct Account {
 /// See https://docs.ftx.com/#get-positions.
 pub type Positions = Vec<Position>;
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Position {
     pub cost: Decimal,
@@ -266,27 +266,27 @@ pub struct Position {
 // REST API -> Wallet
 // TODO
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct WalletDepositAddress {
     pub address: String,
     pub tag: Option<String>,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct WalletBalance {
     pub coin: String,
-    pub free: f64,
-    pub total: f64,
-    pub spot_borrow: f64,
-    pub available_without_borrow: f64,
+    pub free: Decimal,
+    pub total: Decimal,
+    pub spot_borrow: Decimal,
+    pub available_without_borrow: Decimal,
     /// As of 2021-05-12, usdValue is not documented on
     /// https://docs.ftx.com/#get-balances, but it is returned.
-    pub usd_value: Option<f64>,
+    pub usd_value: Option<Decimal>,
 }
 
-#[derive(Debug, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum DepositStatus {
     Confirmed,
@@ -294,15 +294,15 @@ pub enum DepositStatus {
     Cancelled,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct WalletDeposit {
     pub coin: String,
     pub confirmations: usize,
     pub confirmed_time: String,
-    pub fee: f64, // fee, not included in size
-    pub id: usize,
-    pub size: f64,
+    pub fee: Decimal, // fee, not included in size
+    pub id: Id,
+    pub size: Decimal,
     pub status: DepositStatus,
     pub time: String,
     pub txid: Option<String>,
@@ -312,21 +312,21 @@ pub struct WalletDeposit {
 // REST API -> Orders
 // TODO
 
-#[derive(Debug, Serialize, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Serialize, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum OrderType {
     Market,
     Limit,
 }
 
-#[derive(Debug, Serialize, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Serialize, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum OrderSide {
     Buy,
     Sell,
 }
 
-#[derive(Debug, Deserialize, PartialEq)]
+#[derive(Copy, Clone, Debug, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum OrderStatus {
     New, // accepted but not processed yet
@@ -334,21 +334,24 @@ pub enum OrderStatus {
     Closed, // filled or cancelled
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct OrderInfo {
-    pub id: usize,
+    pub id: Id,
     pub market: String,
+    pub future: Option<String>,
     pub r#type: OrderType,
     pub side: OrderSide,
-    pub price: f64,
-    pub size: f64,
-    pub status: OrderStatus,
-    pub filled_size: f64,
-    pub remaining_size: f64,
-    pub created_at: String,
+    pub price: Option<Decimal>, // null for new market orders
+    pub size: Decimal,
     pub reduce_only: bool,
     pub ioc: bool,
     pub post_only: bool,
+    pub status: OrderStatus,
+    pub filled_size: Decimal,
+    pub remaining_size: Decimal,
+    pub avg_fill_price: Option<Decimal>,
+    pub liquidation: Option<bool>,
+    pub created_at: DateTime<Utc>,
     pub client_id: Option<String>,
 }
